#ifndef cpia_h
#define cpia_h

/*#include "cpiaversion.h"*/

enum ioctls
{
// CPiA Module commands
IOCTL_GetCPIAVersion = 0,IOCTL_GetPnPID,IOCTL_GetCameraStatus,
IOCTL_GotoHiPower,IOCTL_GotoLoPower,IOCTL_GotoSuspend,
IOCTL_GotoPassThrough,IOCTL_ModifyCameraStatus,
 
// System Module commands

IOCTL_ReadVCRegs,IOCTL_WriteVCReg,IOCTL_ReadMCPorts,IOCTL_WriteMCPort,
IOCTL_SetBaudRate,IOCTL_SetECPTiming,IOCTL_ReadIDATA,IOCTL_WriteIDATA,
IOCTL_GenericCall,IOCTL_I2CStart,IOCTL_I2cStop,IOCTL_I2cWrite,
IOCTL_I2cRead,

// Video Processor commands

IOCTL_GetVPVersion,IOCTL_SetColourParams,IOCTL_SetExposure,
IOCTL_SetColourBalance,IOCTL_SetSensorFPS,IOCTL_SetVPDefaults,
IOCTL_SetApcor,IOCTL_SetFlickerCtrl,IOCTL_SetVLOffset,
IOCTL_GetColourParameters,IOCTL_GetColourBalance,IOCTL_GetExposure,
IOCTL_SetSensorMatrix,IOCTL_ColourBars,IOCTL_ReadVPRegs,IOCTL_WriteVPReg,

// Capture Control commands

IOCTL_GrabFrame,IOCTL_UploadFrame,IOCTL_SetGrabMode,IOCTL_InitStreamCap,
IOCTL_FiniStreamCap,IOCTL_StartStreamCap,IOCTL_EndStreamCap,
IOCTL_SetFormat,IOCTL_SetROI,IOCTL_SetCompression,IOCTL_SetCompressionTarget,
IOCTL_SetYUVThresh,IOCTL_SetCompressionParams,IOCTL_DiscardFrame,

// Debug Module Commands

IOCTL_OutputRS232,IOCTL_AbortProcess,IOCTL_SetDRAMPage,
IOCTL_StartDRAMUpload,IOCTL_StartDummyStream,IOCTL_AbortStream,
IOCTL_DownloadDRAM,IOCTL_NullCmd,

// High level stuff
IOCTL_EnableStreamMode,
IOCTL_ImageReady,
IOCTL_DisableStreamMode
};

#if IOCTL_GetCPIAVersion != 0
#warning "The first element of the enumeration type isn't zero!"
#endif

//typedef unsigned char cmd_data[4];

struct cmd_data {
  union {
    unsigned char raw[4];
    struct {
      unsigned char StatusByte;
      unsigned char AndMask;
      unsigned char OrMask;
    } ModifyCameraStatus;
    struct {
      unsigned char Addr0;
      unsigned char Addr1;
      unsigned char Addr2;
      unsigned char Addr3;
    } ReadVCRegs; 
    struct {
      unsigned char Addr;
      unsigned char AndMask;
      unsigned char OrMask;
    } WriteVCReg;
    struct {
      unsigned char Port;
      unsigned char AndMask;
      unsigned char OrMask;
    } WriteMCPort;
    struct {
      unsigned char BaudRate;
    } SetBaudRate;
    struct {
      unsigned char SlowTiming;
    } SetECPTiming;
    struct {
      unsigned char Addr0;
      unsigned char Addr1;
      unsigned char Addr2;
      unsigned char Addr3;
    } ReadIDATA;
    struct {
      unsigned char Addr;
      unsigned char AndMask;
      unsigned char OrMask;
    } WriteIDATA;
    struct {
      unsigned char AddrLo;
      unsigned char AddrHi;
    } GenericCall;
    struct {
      unsigned char WriteData;
    } SerialWrite;
    struct {
      unsigned char Brightness;
      unsigned char Contrast;
      unsigned char Saturation;
    } SetColourParams;
    struct {
      unsigned char GainMode;
      unsigned char ExpMode;
      unsigned char CompMode;
      unsigned char CentreWeight;
    } SetExposure;
    struct {
      unsigned char BalanceMode;
      unsigned char RedGain;
      unsigned char GreenGain;
      unsigned char BlueGain;
    } SetColourBalance;
    struct {
      unsigned char SensorClkDivisor;
      unsigned char SensorBaseRate;
      unsigned char PhaseOverride;
      unsigned char NewPhase;
    } SetSensorFPS;
    struct {
    } SetApcor;
    struct {
      unsigned char FlickerMode;
      unsigned char CoarseJump;
      unsigned char AllowableOverExposure;
    } FlickerCtrl;
    struct {
      unsigned char gain1;
      unsigned char gain2;
      unsigned char gain4;
      unsigned char gain8;
    } SetVLOffset;
    struct {
      unsigned char m00;
      unsigned char m01;
      unsigned char m02;
      unsigned char m10;
    } SetSensorMatrix;
    struct {
      unsigned char VPAddr0;
      unsigned char VPAddr1;
      unsigned char VPAddr2;
      unsigned char VPAddr3;
    } ReadVPRegs;
    struct {
      unsigned char VPAddr;
      unsigned char AndMask;
      unsigned char OrMask;
    } WriteVPReg;
    struct {
      unsigned char StreamStartLine;
    } GrabFrame;
    struct {
      unsigned char ForceUpload;
    } UploadFrame;
    struct {
      unsigned char ContinousGrab;
    } SetGrabMode;
    struct {
      unsigned char SkipFrames;
      unsigned char StreamStartLine;
    } InitStreamCap;
    struct {
      unsigned char VideoSize;
      unsigned char SubSample;
      unsigned char YUVOrder;
    } SetFormat;
    struct {
      unsigned char ColStart;
      unsigned char ColEnd;
      unsigned char RowStart;
      unsigned char RowEnd;
    } SetROI;
    struct {
      unsigned char CompMode;
      unsigned char Decimation;
    } SetCompression;
    struct {
      unsigned char FRTargetting;
      unsigned char TargetFR;
      unsigned char TargetQ;
    } SetCompressionTarget;
    struct {
      unsigned char YThresh;
      unsigned char UVThresh;
    } SetYUVThresh;
    struct {
      unsigned char StartPageLo;
      unsigned char StartPageHi;
      unsigned char EndPageLo;
      unsigned char EndPageHi;
    } SetDRAMPage;
  } x;
};

struct ioctlstruct
{
struct cmd_data	*wValue;
unsigned char	*tocam,*fromcam;
};

// Image Stuff

//offsets

#define MAX_IMAGE_SIZE ((352*288*2)+64+(288*3)+256)

#define HEADER_0		0
#define HEADER_1		1

#define TIMESTAME_0		2
#define TIMESTAME_1		3
#define TIMESTAME_2		6
#define TIMESTAME_3		7

#define VIDEO_SIZE		16
#define SUB_SAMPLE		17
#define YUV_ORDER		18

#define COL_START		24
#define COL_END		25
#define ROW_START		26
#define ROW_END		27
#define COMP_ENABLE		28
#define DECIMATION		29
#define Y_THRESH		30
#define UV_THRESH		31
#define CR_ACHIEVED		40
#define FR_ACHIEVED		41

//constant value's

#define MAGIC_0		0x19
#define MAGIC_1		0x68
#define VIDEOSIZE_CIF	1
#define VIDEOSIZE_QCIF	0
#define SUBSAMPLE_420	0
#define SUBSAMPLE_422	1
#define YUVORDER_YUYV	0
#define YUVORDER_UYVY	1
#define NOT_COMPRESSED   0
#define COMPRESSED       1
#define NO_DECIMATION	0
#define DECIMATION_ENAB  1

#define EOI			0xff		// End Of Image
#define EOL			0xfd		// End Of Line

/* Return offsets for GetCameraState */

#define SystemState	0
#define GrabState	1
#define StreamState	2
#define FatalError	3
#define CmdError	4
#define DebugFlags	5
#define VPStatus	6
#define ErrorCode	7

// SystemState

/*#define UNINITIALISED_STATE	1*/
#define LO_POWER_STATE		2
#define HI_POWER_STATE		3

// GrabState

// StreamState

#define STREAM_NOT_READY		0
#define STREAM_READY		1
#define STREAM_OPEN			2
#define STREAM_PAUSED		3
#define STREAM_FINISHED		4

// Fatal Error

// CmdState

// DebugFlags

// VPStatus

// ErrorCode

#endif


